class Edge {
  Node from;
  Node to;
  float[] values = new float[1];
  boolean giveOrReceive;
  float len = 50;
  int count;
  float offset = 0;
  boolean visible = true;
  boolean drawArcLines = false;
  float arcLineOpacity = 0;

  Edge(Node from, Node to, float value, boolean _giveOrReceive) {
    this.from = from;
    this.to = to;
    this.values[0] = value;
    giveOrReceive = _giveOrReceive;
  }

  void increment() {
    count++;
  }

  void addValue(float newValue) {
    values = append(values, newValue);
  }

  void relax() {
    if(visible) {
      float vx = to.x - from.x;
      float vy = to.y - from.y;
      float d = mag(vx, vy);
      if (d > 0) {
        float f = (len - d) / (d * 100);
        float dx = f * vx;
        float dy = f * vy;
        to.dx += dx;
        to.dy += dy;
        from.dx -= dx;
        from.dy -= dy;
      }
    }
  }

  void draw() {

    if(visible) {

      if(layoutMethodToggle) {
        pushMatrix();
        translate(-width/2, -height/2);
      }

      switch(edgeMode) {

      case 1:

        //Thin black lines
        stroke(0);
        strokeWeight(0.3);
        line(from.x, from.y, to.x, to.y);
        break;

      case 2:

        //Blue and red dots
        if(giveOrReceive) {
          stroke(0, 100, 100);
        } 
        else {
          stroke(60, 100, 100);
        }
        //We know start and end points of line x1,y1 and x2,y2
        float angle = atan2((to.y - from.y), (to.x - from.x));    //get the angle of the line
        float lineLength = dist(from.x, from.y, to.x, to.y);      //get the dist() of the line
        float segmentLength = lineLength / count;                 //segmentLength = divide that amount by the # of shapes to be drawn
        for(int i = 1; i < count+1; i++) {                        //loop through, once for each shape:
          float distFromStart = (segmentLength * i) + offset;     //given angle() and ((segmentLength * shape number) + offset)...
          distFromStart = distFromStart % lineLength;
          float px = from.x + distFromStart * cos(angle);         //calculate x/y for where to place shape
          float py = from.y + distFromStart * sin(angle);
          point(px, py);
          offset += offsetIncrement;    //increment offset by some amount (amount determines velocity)
        }
        break;

      case 3:

        //Draw arc lines
        this.drawArcLines();

        //Arcing blue and red dots, all of the same radius
        noStroke();
        if(giveOrReceive) {
          fill(0, 100, 100, 50);
        } 
        else {
          fill(60, 100, 100, 50);
        }
        //We know start and end points of line x1,y1 and x2,y2
        float angle1 = atan2((to.y - from.y), (to.x - from.x));    //get the angle of the line
        float lineLength1 = dist(from.x, from.y, to.x, to.y);      //get the dist() of the line
        float segmentLength1 = lineLength1 / count;                 //segmentLength = divide that amount by the # of shapes to be drawn

        //New matrix so points are drawn along a line from starting x/y to ending x/y
        pushMatrix();
        translate(to.x, to.y);
        rotate(angle1 + PI);

        // Calculate distribution of items -- how many radians per item?
        float radsPerItem = PI / count;
        for(int i = 1; i < count+1; i++) {
          float radsThisItem = (radsPerItem * i + offset) % PI;
          float px = cos(radsThisItem) * lineLength1 / 2 + lineLength1 / 2;
          float py = sin(radsThisItem) * lineLength1 / 2;
          py = py / arcRadiusScale;  //Flatten circle
          ellipse(px, py, r, r);
          offset += offsetIncrement;    //increment offset by some amount (amount of increase here determines velocity)
        }
        popMatrix();
        break;

      case 4:

        //Draw arc lines
        this.drawArcLines();

        //Blue and red dots with different values indicated as size
        noStroke();
        if(giveOrReceive) {
          fill(0, 100, 100, 50);
        } 
        else {
          fill(60, 100, 100, 50);
        }
        //We know start and end points of line x1,y1 and x2,y2
        float angle2 = atan2((to.y - from.y), (to.x - from.x));    //get the angle of the line
        float lineLength2 = dist(from.x, from.y, to.x, to.y);      //get the dist() of the line
        float segmentLength2 = lineLength2 / count;                 //segmentLength = divide that amount by the # of shapes to be drawn

        //New matrix so points are drawn along a line from starting x/y to ending x/y
        pushMatrix();
        translate(to.x, to.y);
        rotate(angle2 + PI);

        // Calculate distribution of items -- how many radians per item?
        float radsPerItem1 = PI / count;
        for(int i = 1; i < count+1; i++) {
          float radsThisItem = (radsPerItem1 * i + offset) % PI;
          float px = cos(radsThisItem) * lineLength2 / 2 + lineLength2 / 2;
          float py = sin(radsThisItem) * lineLength2 / 2;
          py = py / arcRadiusScale;  //Flatten circle
          ellipse(px, py, values[i-1], values[i-1]);
          offset += offsetIncrement;    //increment offset by some amount (amount of increase here determines velocity)
        }
        popMatrix();
        break;

      case 5:

        //Draw arc lines
        this.drawArcLines();

        //Blue and red triangles with different values indicated as size
        noStroke();
        if(giveOrReceive) {
          fill(0, 100, 100, 50);
        } 
        else {
          fill(60, 100, 100, 50);
        }
        //We know start and end points of line x1,y1 and x2,y2
        float angle3 = atan2((to.y - from.y), (to.x - from.x));    //get the angle of the line
        float lineLength3 = dist(from.x, from.y, to.x, to.y);      //get the dist() of the line
        float segmentLength3 = lineLength3 / count;                 //segmentLength = divide that amount by the # of shapes to be drawn

        //New matrix so points are drawn along a line from starting x/y to ending x/y
        pushMatrix();
        translate(to.x, to.y);
        rotate(angle3 + PI);

        // Calculate distribution of items -- how many radians per item?
        float radsPerItem3 = PI / count;
        for(int i = 1; i < count+1; i++) {
          float radsThisItem = (radsPerItem3 * i + offset) % PI;
          float px = cos(radsThisItem) * lineLength3 / 2 + lineLength3 / 2;
          float py = sin(radsThisItem) * lineLength3 / 2;
          py = py / arcRadiusScale;  //Flatten circle
            pushMatrix();
          translate(px, py);
          rotate((atan2(py, px - lineLength3 / 2) - HALF_PI) / arcRadiusScale); //The magical rotation that gets the bars pointed in the right direction
          //rect(px, py, (radsPerItem3 * lineLength3 / 2), values[i-1]);  //Scales width of rects to fit arc length -- weird effect
          rect(0, 0, 5, values[i-1]);
          popMatrix();
          offset += offsetIncrement;    //increment offset by some amount (amount of increase here determines velocity)
        }
        popMatrix();
        break;

      default:
        edgeMode = 1;
        break;
      }

      if(layoutMethodToggle) {
        popMatrix();
      }
    }
  }

  void drawArcLines() {
    //First calculate the angle of the curved line (i.e. half circle) 
    //We already know how wide the curve/circle should be thanks to arcRadiusScale

    //We know start and end points of line x1,y1 and x2,y2
    float angle1 = atan2((to.y - from.y), (to.x - from.x));    //get the angle of the line
    float lineLength1 = dist(from.x, from.y, to.x, to.y);      //get the dist() of the line

    pushMatrix();
    translate(to.x, to.y);
    rotate(angle1 + PI);

    //Adjust guideline opacity
    if(drawArcLines && arcLineOpacity < 25) {
      arcLineOpacity++;
    } 
    else if (!drawArcLines && arcLineOpacity > 0) {
      arcLineOpacity--;
    }

    //Draw guideline
    stroke(0, arcLineOpacity);
    noFill();
    ellipseMode(CENTER);
    ellipse(lineLength1 / 2, 0, lineLength1, lineLength1 / arcRadiusScale);

    //Draw directional arrows
    stroke(0, arcLineOpacity*2);
    int numArrows = 6;
    float radsPerItem5 = PI / numArrows;
    for(int i = 1; i < numArrows+1; i++) {
      float radsThisItem = (radsPerItem5 * i) % PI;
      float px = cos(radsThisItem) * lineLength1 / 2 + lineLength1 / 2;
      float py = sin(radsThisItem) * lineLength1 / 2;
      py = py / arcRadiusScale;  //Flatten circle
      pushMatrix();
      translate(px, py);
      rotate((atan2(py, px - lineLength1 / 2) - HALF_PI) / arcRadiusScale); //The magical rotation that gets the arrows pointed in the right direction
      line(0, 0, 5, 5);
      line(0, 0, 5, -5);
      popMatrix();
    }

    popMatrix();

  }

}







