import processing.core.*; 
import processing.xml.*; 

import processing.pdf.*; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class mcw_vis900600 extends PApplet {

/*
 //  MCW Visualizer
 //  by CSDVA, 12-05-2010
 //
 //  Tested with Processing 1.0
 //
 //  More info: mycenteroftheworld.net
 */





int nodeCount;
Node[] nodes = new Node[100];
HashMap nodeTable = new HashMap();

int edgeCount;
Edge[] edges = new Edge[500];

int edgeMode = 1;
float arcRadiusScale = 30;
float offsetIncrement = 0.01f;
float r = 5;
int minNodeValue = 1;
float easing = 0.1f;

PFont font;
String from;
String to;
float value;
boolean layoutMethodToggle = true;
boolean record;
boolean showHelp = true;



//
//  Setup
//
public void setup() {
  size(900, 600);
  //size(screen.width, screen.height);
  colorMode(HSB, 100);
  font = createFont("SansSerif", 10);
  loadData();
  println("total number of nodes is " + nodeTable.size());
}



//
//  Draw loop
//
public void draw() {
  
  if (mousePressed = true) {
   
      cursor (CROSS);
       }

  //Starts writing this frame to PDF
  if(record) {
    beginRecord(PDF, "mcw_vis-####.pdf");
    colorMode(HSB, 100);
  }

  pushMatrix();
  translate(width/2, height/2);

  background(100);
  textFont(font);
  smooth();

  //If layout method is force-directed
  if(layoutMethodToggle) {
    for (int i = 0; i < edgeCount; i++) {
      edges[i].relax();
    }
    for (int i = 0; i < nodeCount; i++) {
      nodes[i].relax();
    }
    for (int i = 0; i < nodeCount; i++) {
      nodes[i].update();
    }
  } 
  //If layout method is clustering
  else {
    for (int i = 0; i < nodeCount; i++) {
      nodes[i].ease();
    }
  }

  //Draw edges and nodes
  for (int i = 0; i < edgeCount; i++) {
    edges[i].draw();
  }
  for (int i = 0; i < nodeCount; i++) {
    nodes[i].draw();
  }

  popMatrix();

  //Ends writing to PDF
  if(record) {
    endRecord();
    record = false;
  }

  //Draw help
   if (showHelp) {
     String help = "Press h to hide/show\n this help.\n\nUse the mouse to select\n and drag nodes.\nCtrl-click to deselect a node.\nPress 1, 2, 3, 4, and 5 for \ndifferent visualizations.\n[/] decrease/increase the min.\n node value threshold\nL toggles layout method \n(force-directed or clustered)\nJ/K decrease/increase spacing\n (cluster layout only)\nS/F slow/accelerate motion\n (vis. 3, 4, 5, 6 only)\n+/- widen/flatten arcs \n(vis. 3, 4, 5, 6 only)\n0 shows/hides arc guidelines\n (vis. 3, 4, 5, 6 only)\nE/R shrink/grow radii \n(vis. 3 only)";
    float helpWidth = textWidth(help);
    float helpx = 0;
     float helpy = 0;
     fill(52, 9, 99, 80);
    rect(helpx-21, helpy-21, helpWidth+21, 420);
    fill(0);
     textAlign(LEFT, TOP);
     text(help, helpx, helpy);
  }
}





//
//  Loads the data
//
public void loadData() {
  //Reads in a file called "data.csv", which is expected to have rows of three values each, separated by commas only (no quotes):
  //  1)  String --       The label for the "from" node
  //  2)  String --       The label for the "to" node
  //  3)  float or int -- Some arbitrary value that qualifies the nature of the connection

  String[] lines = loadStrings("data.csv");

  //Loops once for each row of data
  for (int i = 0; i < lines.length; i++) {
    String[] values = split(lines[i], ",");
    for (int j = 0; j < values.length; j++) {
      from = values[0];
      to = values[1];
      value = PApplet.parseFloat(values[2]);
    }
    addEdge(from, to, value);
  }
}



//
//  Creates an edge
//
public void addEdge(String fromLabel, String toLabel, float connectionValue) {
  Node from = findNode(fromLabel);
  Node to = findNode(toLabel);
  float value = connectionValue;
  from.increment();
  to.increment();

  //Check to see whether this Edge already exists
  for (int i = 0; i < edgeCount; i++) {
    if (edges[i].from == from && edges[i].to == to) {  //If it does, then...
      edges[i].increment();
      edges[i].addValue(value);
      return;
    }
  }

  //If it doesn't exist yet, then create it, but first check to see if its opposing edge exists, so we can define giveOrReceive
  boolean giveOrReceive = true;
  for (int i = 0; i < edgeCount; i++) {
    if (edges[i].from == to && edges[i].to == from) {  //If from == to and to == from
      giveOrReceive = !edges[i].giveOrReceive;
    }
  }

  Edge e = new Edge(from, to, value, giveOrReceive);
  e.increment();
  if (edgeCount == edges.length) {
    edges = (Edge[]) expand(edges);
  }
  edges[edgeCount++] = e;
}



//
//  Check to see if a node already exists
//
public Node findNode(String label) {
  label = label.toLowerCase();
  Node n = (Node) nodeTable.get(label);
  if (n == null) {
    return addNode(label);
  }
  return n;
}



//
//  Creates a new node
//
public Node addNode(String label) {
  Node n = new Node(label);
  if (nodeCount == nodes.length) {
    nodes = (Node[]) expand(nodes);
  }
  nodeTable.put(label, n);
  nodes[nodeCount++] = n;
  return n;
}



//
//  Handle mouse clicks and drags
//
Node selection;

public void mousePressed() {
  //Ignore anything greater than this distance
  float closest = 20;
  for (int i = 0; i < nodeCount; i++) {
    Node n = nodes[i];
    float d;
    if (layoutMethodToggle) {
      d = dist(mouseX, mouseY, n.x, n.y);
    } 
    else {
      d = dist(mouseX, mouseY, n.x + width/2, n.y + height/2);
    }
    if (d < closest) {
      selection = n;
      closest = d;
    }
  }
  if (selection != null) {
    if (mouseButton == LEFT) {
      selection.fixed = true;
    } 
    else if (mouseButton == RIGHT) {
      selection.fixed = false;
    }
  }
}

public void mouseDragged() {
  if (selection != null) {
    if (layoutMethodToggle) {
      selection.x = mouseX;
      selection.y = mouseY;
    } 
    else {
      selection.x = mouseX - width/2;
      selection.y = mouseY - height/2;
      selection.destx = mouseX - width/2;
      selection.desty = mouseY - height/2;
    }
  }
}

public void mouseReleased() {
  selection = null;
}



//
//  Handle keyboard input
//
public void keyPressed() {
  if (key == '1') {
    edgeMode = 1;
    println("edgeMode now == " + edgeMode);
  } 
  else if (key == '2') {
    edgeMode = 2;
    offsetIncrement = 0.01f;
    println("edgeMode now == " + edgeMode);
  }
  else if (key == '3') {
    edgeMode = 3;
    offsetIncrement = 0.0001f;
    println("edgeMode now == " + edgeMode);
  }
  else if (key == '4') {
    edgeMode = 4;
    offsetIncrement = 0.0001f;
    println("edgeMode now == " + edgeMode);
  }
  else if (key == '5') {
    edgeMode = 5;
    offsetIncrement = 0.0001f;
    println("edgeMode now == " + edgeMode);
  }
  else if (key == 'p') {  //Make PDF
    record = true;
  }
  else if (key == '=') {  //Wider arcs
    if(arcRadiusScale > 1.0f) {
      arcRadiusScale -= 1;
    }
  }
  else if (key == '-') {  //Flatter arcs
  }
  else if (key == 'o' || key == '0') {  //Toggles arc lines on/off
    for(int i = 0; i < edgeCount; i++) {
      edges[i].drawArcLines = !edges[i].drawArcLines;
    }
  }
  else if (key == 'f') {  //Faster motion
    offsetIncrement += 0.0001f;
    println("offsetIncrement now == " + offsetIncrement);
  }
  else if (key == 's') {  //Slower motion
    offsetIncrement -= 0.0001f;
    if (offsetIncrement < 0) {
      offsetIncrement = 0;
    }
    println("offsetIncrement now == " + offsetIncrement);
  }
  else if (key == 'r') {  //Larger edge circles
    r += 0.5f;
  }
  else if (key == 'e') {  //Smaller edge circles
    r -= 0.5f;
  }
  else if (key == 'l') {
    layoutMethodToggle = !layoutMethodToggle;
    if(!layoutMethodToggle) {  //If switching into cluster layout
      clusterAllObjects();
    } 
    else {                   //If switching into force-directed layout
      for(int i = 0; i < nodeCount; i++) {
        nodes[i].x = random(0, width);
        nodes[i].y = random(0, height);
      }
    }
  }
  else if (key == ']') {  //Increase filter threshold by one, and hide nodes below threshold
    minNodeValue++;
    for(int i = 0; i < nodeCount; i++) {
      if(nodes[i].count == minNodeValue - 1) {      //If the node's "count" or sum total of connections is one less than the minNodeValue
        nodes[i].visible = false;              //then hide the node
        for (int j = 0; j < edgeCount; j++) {  //and also loop through and hide any edges that were connected to this node.
          if (edges[j].from.label == nodes[i].label || edges[j].to.label == nodes[i].label) {  //If either the edge's "from" OR "to" points are equal to the node's name (label)
            edges[j].visible = false;                                              //then make the edge invisible.
          }
        }
      }
    }
  }
  else if (key == '[') {  //Decrease filter threshold by one, and reveal nodes above threshold
    minNodeValue--;
    for(int i = 0; i < nodeCount; i++) {
      if(nodes[i].count == minNodeValue) {      //If the node's "count" or sum total of connections is one more than the minNodeValue
        nodes[i].visible = true;              //then reveal the node
        for (int j = 0; j < edgeCount; j++) {  //and also loop through and find any edges that were connected to this node.
          if (edges[j].from.label == nodes[i].label || edges[j].to.label == nodes[i].label) {  //If the edge's "from" OR "to" points are equal to the node's name (label)
            if (edges[j].from.visible && edges[j].to.visible) {  //AND if both nodes are visible
              edges[j].visible = true;  //then make the edge visible.
            }
          }
        }
      }
    }
  }
  else if (key == 'k') {
    if(!layoutMethodToggle) {  //If switching in cluster layout mode
      spacing += 10;
      clusterAllObjects();
    } 
  }
  else if (key == 'j') {
    if(!layoutMethodToggle) {  //If switching in cluster layout mode
      if(spacing > 0) {
        spacing -= 10;
        clusterAllObjects();
      }
    } 
  }
  else if (key == 'h' || key == '/') {
    showHelp = !showHelp;
  }
}
























class Edge {
  Node from;
  Node to;
  float[] values = new float[1];
  boolean giveOrReceive;
  float len = 50;
  int count;
  float offset = 0;
  boolean visible = true;
  boolean drawArcLines = false;
  float arcLineOpacity = 0;

  Edge(Node from, Node to, float value, boolean _giveOrReceive) {
    this.from = from;
    this.to = to;
    this.values[0] = value;
    giveOrReceive = _giveOrReceive;
  }

  public void increment() {
    count++;
  }

  public void addValue(float newValue) {
    values = append(values, newValue);
  }

  public void relax() {
    if(visible) {
      float vx = to.x - from.x;
      float vy = to.y - from.y;
      float d = mag(vx, vy);
      if (d > 0) {
        float f = (len - d) / (d * 100);
        float dx = f * vx;
        float dy = f * vy;
        to.dx += dx;
        to.dy += dy;
        from.dx -= dx;
        from.dy -= dy;
      }
    }
  }

  public void draw() {

    if(visible) {

      if(layoutMethodToggle) {
        pushMatrix();
        translate(-width/2, -height/2);
      }

      switch(edgeMode) {

      case 1:

        //Thin black lines
        stroke(0);
        strokeWeight(0.3f);
        line(from.x, from.y, to.x, to.y);
        break;

      case 2:

        //Blue and red dots
        if(giveOrReceive) {
          stroke(0, 100, 100);
        } 
        else {
          stroke(60, 100, 100);
        }
        //We know start and end points of line x1,y1 and x2,y2
        float angle = atan2((to.y - from.y), (to.x - from.x));    //get the angle of the line
        float lineLength = dist(from.x, from.y, to.x, to.y);      //get the dist() of the line
        float segmentLength = lineLength / count;                 //segmentLength = divide that amount by the # of shapes to be drawn
        for(int i = 1; i < count+1; i++) {                        //loop through, once for each shape:
          float distFromStart = (segmentLength * i) + offset;     //given angle() and ((segmentLength * shape number) + offset)...
          distFromStart = distFromStart % lineLength;
          float px = from.x + distFromStart * cos(angle);         //calculate x/y for where to place shape
          float py = from.y + distFromStart * sin(angle);
          point(px, py);
          offset += offsetIncrement;    //increment offset by some amount (amount determines velocity)
        }
        break;

      case 3:

        //Draw arc lines
        this.drawArcLines();

        //Arcing blue and red dots, all of the same radius
        noStroke();
        if(giveOrReceive) {
          fill(0, 100, 100, 50);
        } 
        else {
          fill(60, 100, 100, 50);
        }
        //We know start and end points of line x1,y1 and x2,y2
        float angle1 = atan2((to.y - from.y), (to.x - from.x));    //get the angle of the line
        float lineLength1 = dist(from.x, from.y, to.x, to.y);      //get the dist() of the line
        float segmentLength1 = lineLength1 / count;                 //segmentLength = divide that amount by the # of shapes to be drawn

        //New matrix so points are drawn along a line from starting x/y to ending x/y
        pushMatrix();
        translate(to.x, to.y);
        rotate(angle1 + PI);

        // Calculate distribution of items -- how many radians per item?
        float radsPerItem = PI / count;
        for(int i = 1; i < count+1; i++) {
          float radsThisItem = (radsPerItem * i + offset) % PI;
          float px = cos(radsThisItem) * lineLength1 / 2 + lineLength1 / 2;
          float py = sin(radsThisItem) * lineLength1 / 2;
          py = py / arcRadiusScale;  //Flatten circle
          ellipse(px, py, r, r);
          offset += offsetIncrement;    //increment offset by some amount (amount of increase here determines velocity)
        }
        popMatrix();
        break;

      case 4:

        //Draw arc lines
        this.drawArcLines();

        //Blue and red dots with different values indicated as size
        noStroke();
        if(giveOrReceive) {
          fill(0, 100, 100, 50);
        } 
        else {
          fill(60, 100, 100, 50);
        }
        //We know start and end points of line x1,y1 and x2,y2
        float angle2 = atan2((to.y - from.y), (to.x - from.x));    //get the angle of the line
        float lineLength2 = dist(from.x, from.y, to.x, to.y);      //get the dist() of the line
        float segmentLength2 = lineLength2 / count;                 //segmentLength = divide that amount by the # of shapes to be drawn

        //New matrix so points are drawn along a line from starting x/y to ending x/y
        pushMatrix();
        translate(to.x, to.y);
        rotate(angle2 + PI);

        // Calculate distribution of items -- how many radians per item?
        float radsPerItem1 = PI / count;
        for(int i = 1; i < count+1; i++) {
          float radsThisItem = (radsPerItem1 * i + offset) % PI;
          float px = cos(radsThisItem) * lineLength2 / 2 + lineLength2 / 2;
          float py = sin(radsThisItem) * lineLength2 / 2;
          py = py / arcRadiusScale;  //Flatten circle
          ellipse(px, py, values[i-1], values[i-1]);
          offset += offsetIncrement;    //increment offset by some amount (amount of increase here determines velocity)
        }
        popMatrix();
        break;

      case 5:

        //Draw arc lines
        this.drawArcLines();

        //Blue and red triangles with different values indicated as size
        noStroke();
        if(giveOrReceive) {
          fill(0, 100, 100, 50);
        } 
        else {
          fill(60, 100, 100, 50);
        }
        //We know start and end points of line x1,y1 and x2,y2
        float angle3 = atan2((to.y - from.y), (to.x - from.x));    //get the angle of the line
        float lineLength3 = dist(from.x, from.y, to.x, to.y);      //get the dist() of the line
        float segmentLength3 = lineLength3 / count;                 //segmentLength = divide that amount by the # of shapes to be drawn

        //New matrix so points are drawn along a line from starting x/y to ending x/y
        pushMatrix();
        translate(to.x, to.y);
        rotate(angle3 + PI);

        // Calculate distribution of items -- how many radians per item?
        float radsPerItem3 = PI / count;
        for(int i = 1; i < count+1; i++) {
          float radsThisItem = (radsPerItem3 * i + offset) % PI;
          float px = cos(radsThisItem) * lineLength3 / 2 + lineLength3 / 2;
          float py = sin(radsThisItem) * lineLength3 / 2;
          py = py / arcRadiusScale;  //Flatten circle
            pushMatrix();
          translate(px, py);
          rotate((atan2(py, px - lineLength3 / 2) - HALF_PI) / arcRadiusScale); //The magical rotation that gets the bars pointed in the right direction
          //rect(px, py, (radsPerItem3 * lineLength3 / 2), values[i-1]);  //Scales width of rects to fit arc length -- weird effect
          rect(0, 0, 5, values[i-1]);
          popMatrix();
          offset += offsetIncrement;    //increment offset by some amount (amount of increase here determines velocity)
        }
        popMatrix();
        break;

      default:
        edgeMode = 1;
        break;
      }

      if(layoutMethodToggle) {
        popMatrix();
      }
    }
  }

  public void drawArcLines() {
    //First calculate the angle of the curved line (i.e. half circle) 
    //We already know how wide the curve/circle should be thanks to arcRadiusScale

    //We know start and end points of line x1,y1 and x2,y2
    float angle1 = atan2((to.y - from.y), (to.x - from.x));    //get the angle of the line
    float lineLength1 = dist(from.x, from.y, to.x, to.y);      //get the dist() of the line

    pushMatrix();
    translate(to.x, to.y);
    rotate(angle1 + PI);

    //Adjust guideline opacity
    if(drawArcLines && arcLineOpacity < 25) {
      arcLineOpacity++;
    } 
    else if (!drawArcLines && arcLineOpacity > 0) {
      arcLineOpacity--;
    }

    //Draw guideline
    stroke(0, arcLineOpacity);
    noFill();
    ellipseMode(CENTER);
    ellipse(lineLength1 / 2, 0, lineLength1, lineLength1 / arcRadiusScale);

    //Draw directional arrows
    stroke(0, arcLineOpacity*2);
    int numArrows = 6;
    float radsPerItem5 = PI / numArrows;
    for(int i = 1; i < numArrows+1; i++) {
      float radsThisItem = (radsPerItem5 * i) % PI;
      float px = cos(radsThisItem) * lineLength1 / 2 + lineLength1 / 2;
      float py = sin(radsThisItem) * lineLength1 / 2;
      py = py / arcRadiusScale;  //Flatten circle
      pushMatrix();
      translate(px, py);
      rotate((atan2(py, px - lineLength1 / 2) - HALF_PI) / arcRadiusScale); //The magical rotation that gets the arrows pointed in the right direction
      line(0, 0, 5, 5);
      line(0, 0, 5, -5);
      popMatrix();
    }

    popMatrix();

  }

}







class Node implements Comparable {
  float x;
  float y;
  float dx;
  float dy;
  float destx;
  float desty;
  boolean fixed;
  String label;
  int count;
  int radius;
  boolean visible = true;
  int spiralPhase = 1;           //Which way we're moving around the spiral --- 1=right, 2=down, 3=left, 4=up
  float spiralMaxDistance = 1;   //The max x or y distance from center

  Node(String label) {
    this.label = label;
    x = random(0, width);
    y = random(0, height);
  }

  public void increment() {
    count++;
  }

  public int compareTo(Object o) 
  { 
    if (!(o instanceof Node)) 
      throw new ClassCastException();
    int oradius = ((Node) o).radius;
    if (oradius < radius)  
      return 1; 
    if (oradius > radius)  
      return -1; 
    return 0; 
  } 

  public void relax() {
    if(visible) {
      float ddx = 0;
      float ddy = 0;

      for (int j = 0; j < nodeCount; j++) {
        Node n = nodes[j];
        if (n != this) {
          float vx = x - n.x;
          float vy = y - n.y;
          float lensq = vx * vx + vy * vy;
          if (lensq == 0) {
            ddx += random(1);
            ddy += random(1);
          } 
          else if (lensq < 1000*1000) {  //orig. 100*100
            ddx += vx / lensq;
            ddy += vy / lensq;
          }
        }
      }
      float dlen = mag(ddx, ddy) / 2;
      if (dlen > 0) {
        dx += ddx / dlen;
        dy += ddy / dlen;
      }
    }
  }

  public void update() {
    if(visible) {
      if (!fixed) {
        x += constrain(dx, -5, 5);
        y += constrain(dx, -5, 5);
      }
      x = constrain(x, 10, width-10);
      y = constrain(y, 10, height-10);
      dx /= 2;
      dy /= 2;
    }
  }

  public void ease() {
    if(visible) {
      // Calculates new x/y/z coords based on target coords and current coords
      // Uses easing factor for smooth animation
      x += (destx - x) * easing;
      y += (desty - y) * easing;
    }
  }

  public void draw() {
    if(visible) {
      fill(80, 80);
      stroke(0);
      strokeWeight(0.3f);

      if(layoutMethodToggle) {
        pushMatrix();
        translate(-width/2, -height/2);
      }

      radius = constrain(count, 1, height/10);
      ellipse(x, y, radius*2, radius*2);
      float w = textWidth(label);

      if (fixed || count > w+2) {
        fill(0);
        textAlign(CENTER, CENTER);
        text(label, x, y);
      }

      if(layoutMethodToggle) {
        popMatrix();
      }
    }
  }

  public void resetObject() {
    x = 0;
    y = 0;
    destx = 0;
    desty = 0;
    spiralPhase = 1;
    spiralMaxDistance = 1;
  }

  public void moveInSpiral() {
    switch(spiralPhase) {
    case 1:
      destx += spacing + 1;  //Move right
      if(destx >= spiralMaxDistance) {
        spiralPhase = 2;  //Switch to next phase
      }
      break;
    case 2:
      desty += spacing + 1;  //Move down
      if(desty >= spiralMaxDistance) {
        spiralPhase = 3;  //Switch to next phase
      }
      break;
    case 3:
      destx -= spacing + 1;  //Move left
      if(abs(destx) >= spiralMaxDistance) {
        spiralPhase = 4;  //Switch to next phase
      }
      break;
    case 4:
      desty -= spacing + 1;  //Move up
      if(abs(desty) >= spiralMaxDistance) {
        spiralPhase = 1;  //Switch to next phase
        spiralMaxDistance++;
      }
      break;
    }
  }
}













//  
//  Circle clustering using a square spiral technique
//


int spacing = 50;


//
//  Trigger threads to cluster all Nodes in background
//
public void clusterAllObjects() {
  println("clusterAllObjects started.");

  Node[] objectsToCluster = new Node[1];       //Create a new object array to store all objects to be clustered
  objectsToCluster[0] = new Node(" ");         //Initialize the first object

  println("node tables size is " + nodeCount);
  for (int i = 0; i < nodeCount; i++) {        //Grab visible objects and throw them in the array
    if (nodes[i].visible) {
      objectsToCluster = (Node[]) append(objectsToCluster, nodes[i]);  //Add this Node to the array of objectsToCluster
    }
  }
  println("number of visible nodes is " + objectsToCluster.length);
  if (objectsToCluster.length > 1) {  //If any were found, then cluster them
    Arrays.sort(objectsToCluster, Collections.reverseOrder());    //Sort objects within array by size
    clusterObjects(objectsToCluster);                             //Cluster objects
  }
}



//
//  Cluster all objects passed to function
//
public void clusterObjects(Node[] objs) {
  println("starting cluster function");

  for(int i = 0; i < objs.length; i++) {  //First reset all positions to 0,0
    objs[i].resetObject();
  }

  //Then resolve collisions --- test current object (i) against all prior objects (j)
  //i starts at 1, though, in order to skip over the first object, which will just remain at 0,0
  for(int i = 1; i < objs.length; i++) {
    boolean collisionsOccurring = true;
    int collisionTally = 0;
    while(collisionsOccurring) {
      for(int j = 0; j < i; j++) {    //Test i vs. all prior j's
        if(isCollision(objs[i], objs[j])) {
          collisionTally++;           //If there is a collision, then increment the tally.
        }
      }
      if(collisionTally > 0) {        //If any collisions happened,
        objs[i].moveInSpiral();       //then move object i to a new position
        collisionTally = 0;           //and start the loop over again.
      } 
      else {                          //But if no collsions happened,
        collisionsOccurring = false;  //then this loop is done, and we can move on to the next object i.
      }
    }
  }
}



//
//  Checks for collisions between any two (circular) objects
//
public boolean isCollision(Node object1, Node object2) {
  float minDistance = object1.radius + object2.radius + spacing;
  float actualDistance = dist(object1.destx, object1.desty, object2.destx, object2.desty);
  boolean isCollision;
  if (actualDistance < minDistance) {
    isCollision = true;
  } 
  else {
    isCollision = false; 
  }
  return isCollision;
}


















  static public void main(String args[]) {
    PApplet.main(new String[] { "--present", "--bgcolor=#666666", "--hide-stop", "mcw_vis900600" });
  }
}
